import { createElement } from 'react';
import dynamic from 'dva/dynamic';
import pathToRegexp from 'path-to-regexp';
import { getMenuData } from './menu';

let routerDataCache;

const modelNotExisted = (app, model) => (
  // eslint-disable-next-line
  !app._models.some(({ namespace }) => {
    return namespace === model.substring(model.lastIndexOf('/') + 1);
  })
);

// wrapper of dynamic
const dynamicWrapper = (app, models, component) => {
  // () => require('module')
  // transformed by babel-plugin-dynamic-import-node-sync
  if (component.toString().indexOf('.then(') < 0) {
    models.forEach((model) => {
      if (modelNotExisted(app, model)) {
        // eslint-disable-next-line
        app.model(require(`../models/${model}`).default);
      }
    });
    return (props) => {
      if (!routerDataCache) {
        routerDataCache = getRouterData(app);
      }
      return createElement(component().default, {
        ...props,
        routerData: routerDataCache,
      });
    };
  }
  // () => import('module')
  return dynamic({
    app,
    models: () => models.filter(
      model => modelNotExisted(app, model)).map(m => import(`../models/${m}.js`)
    ),
    // add routerData prop
    component: () => {
      if (!routerDataCache) {
        routerDataCache = getRouterData(app);
      }
      return component().then((raw) => {
        const Component = raw.default || raw;
        return props => createElement(Component, {
          ...props,
          routerData: routerDataCache,
        });
      });
    },
  });
};

function getFlatMenuData(menus) {
  let keys = {};
  menus.forEach((item) => {
    if (item.children) {
      keys[item.path] = { ...item };
      keys = { ...keys, ...getFlatMenuData(item.children) };
    } else {
      keys[item.path] = { ...item };
    }
  });
  return keys;
}

export const getRouterData = (app) => {
  const routerConfig = {
    '/': {
      component: dynamicWrapper(app, ['user', 'login'], () => import('../layouts/BasicLayout')),
    },
    // 校区管理相关路由注册
    '/campus/list': {
      component: dynamicWrapper(app, ['campus'], () => import('../routes/Campus/CampusList')),
      name: '校区列表',
    },
    '/campus/create': {
      component: dynamicWrapper(app, ['campus'], () => import('../routes/Campus/CampusCreate')),
      name: '添加校区',
    },
    '/campus/edit/:id': {
      component: dynamicWrapper(app, ['campus'], () => import('../routes/Campus/CampusCreate')),
      name: '编辑校区',
    },
    // 终端管理相关路由注册
    '/terminal/user': {
      component: dynamicWrapper(app, ['terminal', 'campus'], () => import('../routes/Terminal/User')),
    },
    '/terminal/user/list': {
      component: dynamicWrapper(app, ['terminal', 'campus'], () => import('../routes/Terminal/User/TerminalList')),
    },
    '/terminal/user/create': {
      component: dynamicWrapper(app, ['terminal', 'campus'], () => import('../routes/Terminal/User/TerminalCreate')),
    },
    '/terminal/user/edit/:id': {
      component: dynamicWrapper(app, ['terminal'], () => import('../routes/Terminal/User/TerminalEdit')),
    },
    '/terminal/auth': {
      component: dynamicWrapper(app, ['terminal'], () => import('../routes/Terminal/Auth')),
    },
    '/terminal/auth/list': {
      component: dynamicWrapper(app, ['terminal'], () => import('../routes/Terminal/Auth/AuthList')),
    },
    // 产品出售相关路由注册
    '/shelves/course': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves')),
    },
    '/shelves/course/list': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves/ShelvesList')),
    },
    '/shelves/course/create': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves/ShelvesCreate')),
    },
    // 查看页面
    '/shelves/course/edit': {
      component: dynamicWrapper(app, ['shelves', 'product', 'resource'], () => import('../routes/Shelves/ShelvesEdit')),
    },
    '/shelves/support': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves')),
    },
    '/shelves/support/list': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves/ShelvesList')),
    },
    '/shelves/support/create': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves/ShelvesCreate')),
    },
    '/shelves/support/edit': {
      component: dynamicWrapper(app, ['shelves', 'resource'], () => import('../routes/Shelves/ShelvesEdit')),
    },
    '/shelves/training': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves')),
    },
    '/shelves/training/list': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves/ShelvesList')),
    },
    '/shelves/training/create': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves/ShelvesCreate')),
    },
    '/shelves/training/edit': {
      component: dynamicWrapper(app, ['shelves', 'resource'], () => import('../routes/Shelves/ShelvesEdit')),
    },
    '/shelves/package': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves')),
    },
    '/shelves/package/list': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves/ShelvesList')),
    },
    '/shelves/package/create': {
      component: dynamicWrapper(app, ['shelves'], () => import('../routes/Shelves/ShelvesCreate')),
    },
    '/shelves/package/edit': {
      component: dynamicWrapper(app, ['shelves', 'resource'], () => import('../routes/Shelves/ShelvesEdit')),
    },
    // 前端管理相关路由注册
    '/frontend/personalize': {
      component: dynamicWrapper(app, ['terminal'], () => import('../routes/Frontend/Personalize')),
    },
    '/frontend/personalize/list': {
      component: dynamicWrapper(app, ['terminal', 'campus'], () => import('../routes/Frontend/Personalize/PersonalizeList')),
    },
    '/frontend/personalize/edit/:id': {
      component: dynamicWrapper(app, ['terminal', 'tag', 'tagType', 'shelves'], () => import('../routes/Frontend/Personalize/PersonalizeEdit')),
    },
    '/trade/shopcart': {
      component: dynamicWrapper(app, [], () => import('../routes/Trade/ShopCart')),
    },
    '/trade/shopcart/list': {
      component: dynamicWrapper(app, ['terminal', 'campus'], () => import('../routes/Trade/ShopCart/ShopCartList')),
    },
    '/trade/shopcart/detail/:id': {
      component: dynamicWrapper(app, ['trade'], () => import('../routes/Trade/ShopCart/ShopCartDetail')),
    },
    '/trade/order': {
      component: dynamicWrapper(app, [], () => import('../routes/Trade/Order')),
    },
    '/trade/order/list': {
      component: dynamicWrapper(app, ['trade'], () => import('../routes/Trade/Order/OrderList')),
    },
    '/trade/order/create': {
      component: dynamicWrapper(app, ['trade', 'terminal', 'shelves'], () => import('../routes/Trade/Order/OrderCreate')),
    },
    '/trade/order/view/:id': {
      component: dynamicWrapper(app, ['trade'], () => import('../routes/Trade/Order/OrderDetail')),
    },
    '/trade/order/sub/:id': {
      component: dynamicWrapper(app, ['trade'], () => import('../routes/Trade/Order/SubOrderDetail')),
    },
    // 统计概览相关路由注册
    // '/dashboard/analysis': {
    //   component: dynamicWrapper(app, ['chart'], () => import('../routes/Dashboard/Analysis')),
    // },
    '/dashboard/sold': {
      component: dynamicWrapper(app, ['trade'], () => import('../routes/Dashboard/SnapshotList')),
    },
    // 异常相关路由注册
    '/exception/403': {
      component: dynamicWrapper(app, [], () => import('../routes/Exception/403')),
    },
    '/exception/404': {
      component: dynamicWrapper(app, [], () => import('../routes/Exception/404')),
    },
    '/exception/500': {
      component: dynamicWrapper(app, [], () => import('../routes/Exception/500')),
    },
    // 登录相关路由注册
    '/user': {
      component: dynamicWrapper(app, [], () => import('../layouts/UserLayout')),
    },
    '/user/login': {
      component: dynamicWrapper(app, ['login'], () => import('../routes/User/Login')),
    },
    // 帮助文档相关路由注册
    '/document/platform': {
      component: dynamicWrapper(app, [], () => import('../routes/Document/DocumentPlatform')),
    },
  };
  // Get name from ./menu.js or just set it in the router data.
  const menuData = getFlatMenuData(getMenuData());

  // Route configuration data
  // eg. {name,authority ...routerConfig }
  const routerData = {};
  // The route matches the menu
  Object.keys(routerConfig).forEach((path) => {
    // Regular match item name
    // eg.  router /user/:id === /user/chen
    const pathRegexp = pathToRegexp(path);
    const menuKey = Object.keys(menuData).find(key => pathRegexp.test(`${key}`));
    let menuItem = {};
    // If menuKey is not empty
    if (menuKey) {
      menuItem = menuData[menuKey];
    }
    let router = routerConfig[path];
    // If you need to configure complex parameter routing,
    // https://github.com/ant-design/ant-design-pro-site/blob/master/docs/router-and-nav.md#%E5%B8%A6%E5%8F%82%E6%95%B0%E7%9A%84%E8%B7%AF%E7%94%B1%E8%8F%9C%E5%8D%95
    // eg . /list/:type/user/info/:id
    router = {
      ...router,
      name: router.name || menuItem.name,
      authority: router.authority || menuItem.authority,
    };
    routerData[path] = router;
  });
  return routerData;
};
