/* eslint-disable react/no-unescaped-entities */
import React, { PureComponent } from 'react';
import { Card, Icon } from 'antd';
import DescriptionList from '../../components/DescriptionList';
import styles from './document.less';

const { Description } = DescriptionList;

const productQAList = [{
  question: '查看已上架产品',
  answer: '操作："产品库 > 产品类型(课程|配套...) > 查看"，点击查看可查看产品详情内容、价格、上架到的栏目等信息',
  imgList: [
    'http://efunimgs.oss-cn-beijing.aliyuncs.com/resources/document/0101/15308557800763008.png',
  ],
}];
const frontendQAList = [{
  question: '个性化配置',
  answer: '操作："前端配置 > 个性化配置 > 用户配置"',
  note: '进入用户配置界面后可以看到该用户在各类型标签(配套[SUPPORT]，特色课[STYLE]...)下的侧边栏目(被称作标签), 点击创建/编辑即可配置该栏目内容',
  imgList: [
    'http://efunimgs.oss-cn-beijing.aliyuncs.com/resources/document/0201/15308564463242998.png',
    'http://efunimgs.oss-cn-beijing.aliyuncs.com/resources/document/0202/15308571570325302.png',
  ],
}];
const tradeQAList = [{
  question: '交易管理',
  answer: '操作："交易管理 > 购物车|订单列表 > 创建"',
  note: '可以根据终端发送的购物车内容快速创建订单，也可在订单列表界面手动生成订单，支付成功后即可为账号增加对应产品的使用权限',
  imgList: [
    'http://efunimgs.oss-cn-beijing.aliyuncs.com/resources/document/0301/15308567509614869.png',
    'http://efunimgs.oss-cn-beijing.aliyuncs.com/resources/document/0302/15308567873894538.png',
  ],
}];
const otherQAList = [{
  question: '校区及终端用户管理',
  answer: '校区管理、终端管理可以创建校区及对应校区下的终端用户，可以进行账号禁用、解绑等相关操作',
  imgList: [
    'http://efunimgs.oss-cn-beijing.aliyuncs.com/resources/document/0401/15308569206579359.png',
    'http://efunimgs.oss-cn-beijing.aliyuncs.com/resources/document/0402/1530856943105645.png',
  ],
}, {
  question: '查看用户权限',
  answer: '点击权限列表可以查找相关账号购买的产品包及使用有效期限等信息',
  imgList: [
    'http://efunimgs.oss-cn-beijing.aliyuncs.com/resources/document/0403/15308570742061372.png',
  ],
}];

function FAQComponent({ sort, bigTitle, qaList }) {
  return (
    <DescriptionList
      col={1}
      size="large"
      title={`${sort}.${bigTitle}`}
      className={styles.descList}
    >
      {/* eslint-disable react/no-array-index-key */}
      {
        qaList.map((qa, index) => {
          const { question, answer, note, imgList } = qa;
          return (
            <div key={index}>
              <Description>
                <Icon type="question-circle-o" className={styles.question} />&nbsp;{question}
              </Description>
              <Description>
                <Icon type="check-circle-o" className={styles.answer} />&nbsp;{answer}
                {note && (<div>&nbsp;&nbsp;&nbsp;&nbsp;{note}</div>)}
              </Description>
              {(imgList && imgList.length) && imgList.map((img, imgIndex) => (
                <Description key={imgIndex}>
                  <img src={img} alt="" style={{ width: '100%', height: 550 }} />
                </Description>
              ))}
            </div>
          );
        })
      }
    </DescriptionList>
  );
}

export default class PlatformDocument extends PureComponent {
  render() {
    return (
      <Card title="平台使用文档" className={styles.card}>
        <FAQComponent
          sort={1}
          bigTitle="产品库"
          qaList={productQAList}
        />
        <FAQComponent
          sort={2}
          bigTitle="前端配置"
          qaList={frontendQAList}
        />
        <FAQComponent
          sort={3}
          bigTitle="交易管理"
          qaList={tradeQAList}
        />
        <FAQComponent
          sort={5}
          bigTitle="其他问题"
          qaList={otherQAList}
        />
      </Card>
    );
  }
}
